@@ -4,19 +4,24 @@ The main DebugToolbar class that loads and renders the Toolbar.
 
 import uuid
 from collections import OrderedDict
+from functools import lru_cache
 
 from django.apps import apps
 from django.core.exceptions import ImproperlyConfigured
+from django.dispatch import Signal
 from django.template import TemplateSyntaxError
 from django.template.loader import render_to_string
 from django.urls import path, resolve
 from django.urls.exceptions import Resolver404
 from django.utils.module_loading import import_string
 
-from debug_toolbar import settings as dt_settings
+from debug_toolbar import APP_NAME, settings as dt_settings
 
 
 class DebugToolbar:
+    # for internal testing use only
+    _created = Signal()
+
     def __init__(self, request, get_response):
         self.request = request
         self.config = dt_settings.get_config().copy()
@@ -27,6 +32,9 @@ class DebugToolbar:
             if panel.enabled:
                 get_response = panel.process_request
         self.process_request = get_response
+        # Use OrderedDict for the _panels attribute so that items can be efficiently
+        # removed using FIFO order in the DebugToolbar.store() method.  The .popitem()
+        # method of Python's built-in dict only supports LIFO removal.
         self._panels = OrderedDict()
         while panels:
             panel = panels.pop()
@@ -34,6 +42,7 @@ class DebugToolbar:
         self.stats = {}
         self.server_timing_stats = {}
         self.store_id = None
+        self._created.send(request, toolbar=self)
 
     # Manage panels
 
@@ -130,7 +139,7 @@ class DebugToolbar:
             # Load URLs in a temporary variable for thread safety.
             # Global URLs
             urlpatterns = [
-                path("render_panel/", views.render_panel, name="render_panel")
+                path("render_panel/", views.render_panel, name="render_panel"),
             ]
             # Per-panel URLs
             for panel_class in cls.get_panel_classes():
@@ -143,8 +152,6 @@ class DebugToolbar:
         """
         Determine if the request is for a DebugToolbar view.
         """
-        from debug_toolbar.urls import app_name
-
         # The primary caller of this function is in the middleware which may
         # not have resolver_match set.
         try:
@@ -153,4 +160,22 @@ class DebugToolbar:
             )
         except Resolver404:
             return False
-        return resolver_match.namespaces and resolver_match.namespaces[-1] == app_name
+        return resolver_match.namespaces and resolver_match.namespaces[-1] == APP_NAME
+
+    @staticmethod
+    @lru_cache(maxsize=128)
+    def get_observe_request():
+        # If OBSERVE_REQUEST_CALLBACK is a string, which is the recommended
+        # setup, resolve it to the corresponding callable.
+        func_or_path = dt_settings.get_config()["OBSERVE_REQUEST_CALLBACK"]
+        if isinstance(func_or_path, str):
+            return import_string(func_or_path)
+        else:
+            return func_or_path
+
+
+def observe_request(request):
+    """
+    Determine whether to update the toolbar from a client side request.
+    """
+    return not DebugToolbar.is_toolbar_request(request)
